@@ -12,19 +12,26 @@
 #include "iop-profiles.h"
 #include "orb-core-private.h"
 #include "orbhttp.h"
+#include "orbit-debug.h"
 
 extern ORBit_option orbit_supported_options[];
 
+#ifdef G_ENABLE_DEBUG
+OrbitDebugFlags _orbit_debug_flags = ORBIT_DEBUG_NONE;
+#endif
+
 /*
  * Command line option handling.
  */
-static gboolean     orbit_use_ipv4   = FALSE;
-static gboolean     orbit_use_ipv6   = FALSE; 
-static gboolean     orbit_use_usocks = TRUE;
-static gboolean     orbit_use_irda   = FALSE;
-static gboolean     orbit_use_ssl    = FALSE;
-static char        *orbit_ipsock     = NULL;
-static char        *orbit_ipname     = NULL;
+static gboolean     orbit_use_ipv4          = FALSE;
+static gboolean     orbit_use_ipv6          = FALSE; 
+static gboolean     orbit_use_usocks        = TRUE;
+static gboolean     orbit_use_irda          = FALSE;
+static gboolean     orbit_use_ssl           = FALSE;
+static gboolean     orbit_use_genuid_simple = FALSE;
+static char        *orbit_ipsock            = NULL;
+static char        *orbit_ipname            = NULL;
+static char        *orbit_debug_options     = NULL;
 
 void
 ORBit_ORB_start_servers (CORBA_ORB orb)
@@ -78,6 +85,21 @@ ORBit_ORB_shutdown_servers (CORBA_ORB orb)
 	orb->servers = NULL;
 }
 
+static ORBitGenUidType
+ORBit_genuid_type (void)
+{
+	ORBitGenUidType retval = ORBIT_GENUID_STRONG;;
+
+	if (orbit_use_genuid_simple)
+		retval = ORBIT_GENUID_SIMPLE;
+
+	else if (orbit_use_usocks && !orbit_use_ipv4 &&
+		 !orbit_use_ipv6 && !orbit_use_irda)
+		retval = ORBIT_GENUID_SIMPLE;
+
+	return retval;
+}
+
 static void
 ORBit_service_list_free_ref (gpointer         key,
 			     ORBit_RootObject objref,
@@ -106,7 +128,37 @@ ORBit_locks_initialize (void)
 	ORBit_RootObject_lifecycle_lock = linc_mutex_new ();
 }
 
+#ifdef G_ENABLE_DEBUG
+static void
+ORBit_setup_debug_flags (void)
+{
+	static GDebugKey debug_keys[] = {
+		{ "traces",   ORBIT_DEBUG_TRACES },
+		{ "timings",  ORBIT_DEBUG_TIMINGS },
+		{ "types",    ORBIT_DEBUG_TYPES },
+		{ "messages", ORBIT_DEBUG_MESSAGES },
+		{ "objects",  ORBIT_DEBUG_OBJECTS },
+	};
+	const char *env_string;
+
+	env_string = g_getenv ("ORBIT2_DEBUG");
+
+	if (env_string)
+		_orbit_debug_flags |=
+			g_parse_debug_string (env_string,
+					      debug_keys,
+					      G_N_ELEMENTS (debug_keys));
+
+	if (orbit_debug_options)
+		_orbit_debug_flags |=
+			g_parse_debug_string (orbit_debug_options,
+					      debug_keys,
+					      G_N_ELEMENTS (debug_keys));
+}
+#endif /* G_ENABLE_DEBUG */
+
 static CORBA_ORB _ORBit_orb = NULL;
+static gulong    init_level = 0;
 
 CORBA_ORB
 CORBA_ORB_init (int *argc, char **argv,
@@ -119,6 +171,8 @@ CORBA_ORB_init (int *argc, char **argv,
 		CORBA_ORB_release_fn
 	};
 
+	init_level++;
+
 	if ((retval = _ORBit_orb))
 		return ORBit_RootObject_duplicate (retval);
 
@@ -127,6 +181,10 @@ CORBA_ORB_init (int *argc, char **argv,
 
 	ORBit_option_parse (argc, argv, orbit_supported_options);
 
+#ifdef G_ENABLE_DEBUG
+	ORBit_setup_debug_flags ();
+#endif /* G_ENABLE_DEBUG */
+
 	giop_init ();
 
 	ORBit_locks_initialize ();
@@ -137,7 +195,7 @@ CORBA_ORB_init (int *argc, char **argv,
 	/* released by CORBA_ORB_destroy */
 	_ORBit_orb = ORBit_RootObject_duplicate (retval);
 
-	ORBit_genrand_init (&retval->genrand);
+	ORBit_genuid_init (ORBit_genuid_type ());
 	retval->default_giop_version = GIOP_LATEST;
 
 	retval->adaptors = g_ptr_array_new ();
@@ -448,6 +506,31 @@ CORBA_ORB_create_struct_tc (CORBA_ORB                    _obj,
   return NULL;
 }
 
+static void
+copy_case_value (CORBA_long *dest,
+		 CORBA_any  *src)
+{
+        switch (src->_type->kind) {
+        case CORBA_tk_ulong:
+        case CORBA_tk_long:
+        case CORBA_tk_enum:
+                *dest = *(CORBA_long *) src->_value;
+                break;
+        case CORBA_tk_ushort:
+        case CORBA_tk_short:
+                *dest = *(CORBA_short *) src->_value;
+                break;
+        case CORBA_tk_char:
+        case CORBA_tk_boolean:
+        case CORBA_tk_octet:
+                *dest = *(CORBA_octet *) src->_value;
+                break;
+        default:
+		g_assert_not_reached ();
+		break;
+        }
+}
+
 CORBA_TypeCode
 CORBA_ORB_create_union_tc (CORBA_ORB                   _obj,
 			   const CORBA_char           *id,
@@ -474,7 +557,7 @@ CORBA_ORB_create_union_tc (CORBA_ORB                   _obj,
   if(tc->subnames==NULL)
     goto subnames_alloc_failed;
 
-  tc->sublabels=g_new0(CORBA_any, members->_length);
+  tc->sublabels=g_new0(CORBA_long, members->_length);
   if(tc->sublabels == NULL)
     goto sublabels_alloc_failed;
 
@@ -489,7 +572,7 @@ CORBA_ORB_create_union_tc (CORBA_ORB                   _obj,
     CORBA_UnionMember *mem=(CORBA_UnionMember *)&(members->_buffer[i]);
 
     g_assert(&(mem->label)!=NULL);
-    memcpy(&(tc->sublabels[i]), &(mem->label), (size_t)sizeof(CORBA_any));
+    copy_case_value (&tc->sublabels [i], &mem->label);
     g_assert(&(mem->type)!=NULL);
     tc->subtypes[i] = ORBit_RootObject_duplicate(mem->type);
     tc->subnames[i]=g_strdup(mem->name);
@@ -820,7 +903,7 @@ CORBA_ORB_create_array_tc(CORBA_ORB _obj,
 }
 
 CORBA_TypeCode
-CORBA_ORB_create_value_tc (CORBA_ORB                   _obj,
+CORBA_ORB_create_value_tc (CORBA_ORB                   obj,
 			   const CORBA_char           *id,
 			   const CORBA_char           *name,
 			   const CORBA_ValueModifier   type_modifier,
@@ -832,26 +915,26 @@ CORBA_ORB_create_value_tc (CORBA_ORB                   _obj,
 }
 
 CORBA_TypeCode
-CORBA_ORB_create_value_box_tc (CORBA_ORB                 _obj,
-			       const CORBA_char         *id,
-			       const CORBA_char         *name,
-			       const CORBA_TypeCode      boxed_type,
-			       CORBA_Environment        *ev)
+CORBA_ORB_create_value_box_tc (CORBA_ORB            obj,
+			       const CORBA_char    *id,
+			       const CORBA_char    *name,
+			       const CORBA_TypeCode boxed_type,
+			       CORBA_Environment   *ev)
 {
   return CORBA_OBJECT_NIL;
 }
 
 CORBA_TypeCode
-CORBA_ORB_create_native_tc (CORBA_ORB                 _obj,
-			    const CORBA_char         *id,
-			    const CORBA_char         *name,
-			    CORBA_Environment        *ev)
+CORBA_ORB_create_native_tc (CORBA_ORB          obj,
+			    const CORBA_char  *id,
+			    const CORBA_char  *name,
+			    CORBA_Environment *ev)
 {
   return CORBA_OBJECT_NIL;
 }
 
 CORBA_TypeCode
-CORBA_ORB_create_recursive_tc (CORBA_ORB          _obj,
+CORBA_ORB_create_recursive_tc (CORBA_ORB          obj,
 			       const CORBA_char  *id,
 			       CORBA_Environment *ev)
 {
@@ -859,25 +942,26 @@ CORBA_ORB_create_recursive_tc (CORBA_ORB          _obj,
 }
 
 CORBA_TypeCode
-CORBA_ORB_create_abstract_interface_tc (CORBA_ORB                 _obj,
-				        const CORBA_char         *id,
-				        const CORBA_char         *name,
-				        CORBA_Environment        *ev)
+CORBA_ORB_create_abstract_interface_tc (CORBA_ORB          obj,
+				        const CORBA_char  *id,
+				        const CORBA_char  *name,
+				        CORBA_Environment *ev)
 {
   return CORBA_OBJECT_NIL;
 }
 
 CORBA_boolean
-CORBA_ORB_work_pending(CORBA_ORB _obj,
-		       CORBA_Environment * ev)
+CORBA_ORB_work_pending (CORBA_ORB          obj,
+			CORBA_Environment *ev)
 {
-  return linc_main_pending();
+  return linc_main_pending ();
 }
 
 void
-CORBA_ORB_perform_work(CORBA_ORB _obj, CORBA_Environment * ev)
+CORBA_ORB_perform_work (CORBA_ORB          obj,
+			CORBA_Environment *ev)
 {
-  linc_main_iteration(FALSE);
+	linc_main_iteration (FALSE);
 }
 
 void
@@ -888,12 +972,15 @@ CORBA_ORB_run (CORBA_ORB          orb,
 }
 
 void
-CORBA_ORB_shutdown (CORBA_ORB            orb,
-		    const CORBA_boolean  wait_for_completion,
-		    CORBA_Environment   *ev)
+CORBA_ORB_shutdown (CORBA_ORB           orb,
+		    const CORBA_boolean wait_for_completion,
+		    CORBA_Environment  *ev)
 {
 	PortableServer_POA root_poa;
 
+	if (init_level > 0)
+		return;
+
 	root_poa = g_ptr_array_index (orb->adaptors, 0);
 	if (root_poa) {
 		PortableServer_POA_destroy (
@@ -915,12 +1002,17 @@ CORBA_ORB_destroy (CORBA_ORB          orb,
 {
 	PortableServer_POA root_poa;
 
-	g_assert (_ORBit_orb == orb);
-	_ORBit_orb = NULL;
-
 	if (orb->life_flags & ORBit_LifeF_Destroyed)
 		return;
 
+	init_level--;
+
+	if (init_level > 0)
+		return;
+
+	g_assert (_ORBit_orb == orb);
+	_ORBit_orb = NULL;
+
 	CORBA_ORB_shutdown (orb, TRUE, ev);
 	if (ev->_major)
 		return;
@@ -944,27 +1036,27 @@ CORBA_ORB_destroy (CORBA_ORB          orb,
 
 	{
 		int i;
-		int leaked_poas = 0;
+		int leaked_adaptors = 0;
 
 		/* Each poa has a ref on the ORB */
 		for (i = 0; i < orb->adaptors->len; i++) {
-			PortableServer_POA poa;
+			ORBit_ObjectAdaptor adaptor;
 
-			poa = g_ptr_array_index (orb->adaptors, i);
+			adaptor = g_ptr_array_index (orb->adaptors, i);
 
-			if (poa)
-				leaked_poas++;
+			if (adaptor)
+				leaked_adaptors++;
 		}
 
-		if (leaked_poas) {
-			g_warning ("CORBA_ORB_destroy: leaked '%d' POAs", leaked_poas);
+		if (leaked_adaptors) {
+			g_warning ("CORBA_ORB_destroy: leaked '%d' Object Adaptors", leaked_adaptors);
 			CORBA_exception_set_system (
 				ev, ex_CORBA_FREE_MEM, CORBA_COMPLETED_NO);
 		}
 
-		if (((ORBit_RootObject)orb)->refs != 2 + leaked_poas) {
+		if (((ORBit_RootObject)orb)->refs != 2 + leaked_adaptors) {
 			g_warning ("CORBA_ORB_destroy: ORB still has %d refs.",
-				   ((ORBit_RootObject)orb)->refs - 1 - leaked_poas);
+				   ((ORBit_RootObject)orb)->refs - 1 - leaked_adaptors);
 			CORBA_exception_set_system (
 				ev, ex_CORBA_FREE_MEM, CORBA_COMPLETED_NO);
 		}
@@ -1045,7 +1137,7 @@ ORBit_proto_use (const char *name)
 	    (orbit_use_ipv6   && !strcmp ("IPv6", name)) || 
 	    (orbit_use_usocks && !strcmp ("UNIX", name)) || 
 	    (orbit_use_irda   && !strcmp ("IrDA", name)) || 
-	    (orbit_use_ipv6   && !strcmp ("SSL",  name)))
+	    (orbit_use_ssl    && !strcmp ("SSL",  name)))
 		return TRUE;
 
 	return FALSE;
@@ -1065,6 +1157,8 @@ static ORBit_option orbit_supported_options[] = {
 	{"ORBIIOPUNIX",     ORBIT_OPTION_BOOLEAN, &orbit_use_usocks},
 	{"ORBIIOPIrDA",     ORBIT_OPTION_BOOLEAN, &orbit_use_irda},
 	{"ORBIIOPSSL",      ORBIT_OPTION_BOOLEAN, &orbit_use_ssl},
+	{"ORBSimpleUIDs",   ORBIT_OPTION_BOOLEAN, &orbit_use_genuid_simple},
+	{"ORBDebugFlags",   ORBIT_OPTION_STRING,  &orbit_debug_options},
 	{NULL,              0,                    NULL},
 };
 
